<?php

namespace App\Exceptions;

use App\Http\Controllers\Base;
use Exception;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Foundation\Exceptions\Handler as ExceptionHandler;
use Illuminate\Http\Exceptions\ThrottleRequestsException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Throwable;

class Handler extends ExceptionHandler
{
    /**
     * A list of exception types with their corresponding custom log levels.
     *
     * @var array<class-string<\Throwable>, \Psr\Log\LogLevel::*>
     */
    protected $levels = [
        //
    ];

    /**
     * A list of the exception types that are not reported.
     *
     * @var array<int, class-string<\Throwable>>
     */
    protected $dontReport = [
        //
    ];

    /**
     * A list of the inputs that are never flashed to the session on validation exceptions.
     *
     * @var array<int, string>
     */
    protected $dontFlash = [
        'current_password',
        'password',
        'password_confirmation',
    ];

    /**
     * Register the exception handling callbacks for the application.
     *
     * @return void
     */
    public function register()
    {
        $this->renderable(function (Exception $exception, $request) {
                if ($exception instanceof HttpException && $exception->getStatusCode() == 429) {
                return response()->json([
                    'success' => false,
                    'code' => 429,
                    'message' => "Too many wrong attempt, please try again later.",
                    'data' => [],
                    'type' => 'exception'
                ], 429)->withHeaders($exception->getHeaders());
                    // return response()->json([
                    //     'message' => 'Too Many Attempts',
                    //     'code' => 429
                    // ], 429)->withHeaders($exception->getHeaders());
                }});
    }

    public function unauthenticated($request, AuthenticationException $exception){
        if(!$request->expectsJson()){
            if(collect($request->route()->middleware())->contains('api')) {
                return Base::error("You are not logged in.", [], 'not_auth');
            }
        }
    }

    /**
     * Render an exception into an HTTP response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Exception  $exception
     * @return \Illuminate\Http\Response|\Illuminate\Http\JsonResponse
     */
    // public function render($request, Exception $exception)
    // {
    //     if ($exception instanceof HttpException && $exception->getStatusCode() == 429) {
    //         return response()->json([
    //             'message' => 'Too Many Attempts',
    //             'code' => 429
    //         ], 429)->withHeaders($exception->getHeaders());
    //     }

    //     return parent::render($request, $exception);
    // }
}
