<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Base;
use App\Http\Controllers\Controller;
use App\Http\Requests\AdminAddCustomerRequest;
use App\Models\User;
use App\Services\Interface\UserInterface;
use Exception;
use Illuminate\Http\Request;

class CustomerController extends Controller
{
    private UserInterface $userInterface;

    public function __construct(UserInterface $userRepository)
    {
        $this->userRepository = $userRepository;
    }
    //
    public function index(Request $request)
    {
        $data = $this->userRepository->adminCustomerIndex($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }
    public function adminCustomerFilter(Request $request)
    {
        $data = $this->userRepository->adminCustomerFilter($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }
    public function show(Request $request)
    {
        $data = $this->userRepository->adminCustomerShow($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }
    public function statusToggle(Request $request)
    {
        $data = $this->userRepository->adminStatusToggle($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }

    public function vidStatusToggle(Request $request)
    {
        $data = $this->userRepository->adminVidStatusToggle($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }

    public function update(Request $request)
    {
        try{
            $data = $this->userRepository->adminUserUpdate($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }

    public function adminCustomerChangePassword(Request $request){
        $data = $this->userRepository->adminUserUpdate($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }
    
    public function addCustomer(AdminAddCustomerRequest $request)
    {
        try{
            $data = $this->userRepository->adminAddCustomer($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }
    public function adminDeleteCustomer(UserInterface $i_user,Request $request)
    {
        try {

            $user = User::find($request->user_id);
            if(!$user) return Base::error("User not found"); //DONE

            $res = $i_user->delete($user);
            if (!$res->success) return Base::error($res->message);

            // $token = Auth::user()->token();
            // $token->revoke();
            $user->tokens->each(function ($token, $key) {
                $token->delete();
            });

            // $refreshTokenRepository = app(\Laravel\Passport\RefreshTokenRepository::class);
            // foreach (User::find($user->id)->tokens as $token) {
            //     $token->revoke();
            //     $refreshTokenRepository->revokeRefreshTokensByAccessTokenId($token->id);
            // }

            return Base::success('User Deleted', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

}
