<?php

namespace App\Http\Controllers;

use App\Http\Requests\PostCardIDRequest;
use App\Models\Category;
use App\Models\Order;
use App\Models\PostCard;
use App\Models\User;
use App\Services\Interface\CategoryInterface;
use App\Services\Interface\PostCardInterface;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;

use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Validator;

class PostCardController extends Controller
{
    public function index(PostCardInterface $i_post_card,CategoryInterface $i_category, Request $request) {
        try {
            $categories = $i_post_card->getCategories();
            if(!$categories->success) return Base::error($categories->message);

            // return
            $route_name = Route::currentRouteName();
            if($route_name == 'common'){
                $post_cards = $i_post_card->filter(Auth::id(), $request);
            }
            if($route_name == "shop_cards"){
                $post_cards = $i_post_card->filter($request->shop_id, $request);
            }
            if($route_name == "get.recommended.card"){
                $get_recommended_category = $i_category->getRecommendedCategoryList();
                if (!$get_recommended_category->success) return Base::error($get_recommended_category->message);
                if ($get_recommended_category->data == null) return Base::error("No recommended cards"); // DONE
                $category = [];
                $get_recommended_category_data = $get_recommended_category->data;
                foreach($get_recommended_category_data as $item){
                    // return $item['category_id'];
                    array_push($category, $item['category_id']);
                }
                $request['categories'] = $category;
                $post_cards = $i_post_card->filter(null, $request);
            }

            if($route_name == "get.popular.card"){
                $request['type'] = "popular_cards";
                $post_cards = $i_post_card->filter(null, $request);
            }
            if($route_name == "get.new.card"){
                $request['type'] = "new_cards";
                $post_cards = $i_post_card->filter(null, $request);
            }

            if(!$post_cards->success) return Base::error($post_cards->message);
            if(isset($request->shop_id)){
                $new_post_card = PostCard::where('shop_id', $request->shop_id)->where('created_at', '>=', Carbon::now()->subDay(2))->get()->count();
            }
            else{
                $new_post_card = PostCard::where('created_at', '>=', Carbon::now()->subDay())->get()->count();
            }

            $data = [
                'post_cards' => $post_cards->data,
                'categories' => $categories->data,
                'new_card_count' => $new_post_card
            ];
            
            return Base::success('', $data);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function show(PostCardInterface $i_post_card, Request $request) {
        try {
            $validator =  Validator::make($request->all(), [
                'id' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);
            
            $res = $i_post_card->show($request->id);
            if(!$res->success) return Base::error($res->message);

            return Base::success('Post card details fetched', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }


    public function add(PostCardInterface $i_post_card, Request $request) {
        try {
            if(!isset($request->post_cards)) return Base::error("Post card data is empty"); // DONE

            foreach ($request->post_cards as $item) {
                $validator =  Validator::make($item, [
                    'category_id' => 'required',
                    'name' => 'required',
                    // 'code' => 'required|unique:post_cards',
                    'price' => 'required',
                ]);
                if ($validator->fails()) return Base::validation($validator);

                $category = Category::find($item['category_id']);
                if(!$category) return Base::error("Category is not found"); // DONE
            }

            $data = [];

            foreach ($request->post_cards as $item) {
                $item['shop_id'] = isset($item['shop_id']) ? $item['shop_id'] : Auth::id();
                $res = $i_post_card->add($item);
                if(!$res->success) return Base::error($res->message);
                array_push($data, $res->data);
            }

            return Base::success('Post Cards Added', $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function update(PostCardInterface $i_post_card, Request $request) {
        try {
            $validator =  Validator::make($request->all(), [
                'id' => 'required',
                // 'category_id' => 'required',
                // 'name' => 'required',
                // 'code' => 'required',
                // 'price' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);
            $post_card = PostCard::where([['id', $request->id]])->first();
            // $post_card = PostCard::where([['id', $request->id], ['shop_id', Auth::id()]])->first();
            if (!$post_card) return Base::fail("Post Card not found"); // DONE

            if(isset($request->category_id)){
                $category = Category::find($request->category_id);
                if (!$category) return Base::error("Category is not found"); //Done
            }else $request->category_id = $post_card->category_id;

            if (isset($request->shop_id)) {
                $shop_id = User::find($request->shop_id);
                if (!$shop_id) return Base::error("Shop not found"); // DONE
            } else $request->shop_id = $post_card->shop_id;

            if(!isset($request->name)){
                $request->name = $post_card->name;
            }
            if(isset($request->code)){
                $is_code_duplicate = PostCard::where([['code', $request->code], ['id', '!=', $request->id]])->first();
                if ($is_code_duplicate) return Base::error("Code already taken"); // DONE
            }
            else $request->code = $post_card->code;
            if (!isset($request->price)) {
                $request->price = $post_card->price;
            }

            if(isset($request->shipping_cost)){
                $request->shipping_cost = $request->shipping_cost;
            }
            else $request->shipping_cost = $post_card->shipping_cost;

            $res = $i_post_card->update($post_card, $request->all());
            if(!$res->success) return Base::error($res->message);

            return Base::success('Post Cards Updated', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function delete(PostCardInterface $i_post_card, Request $request) {
        try {
            $validator =  Validator::make($request->all(), [
                'id' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $shop_id = isset($request->shop_id) ? $request->shop_id : Auth::id() ;

            $post_card = PostCard::where([['id', $request->id], ['shop_id', $shop_id]])->first();
            if(!$post_card) return Base::error("Post Card not found"); // DONE

            $order = Order::where(['card_id' => $request->id])->whereIn('status', ['payment_due', 'pending', 'processing'])->get();
            if($order->isNotEmpty()) return Base::error("Can not Delete, This card has pending order"); // DONE

            $res = $i_post_card->delete($post_card);
            if(!$res->success) return Base::error($res->message);

            return Base::success('Post Cards Deleted'); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function statusToggle(PostCardInterface $i_post_card, PostCardIDRequest $request)
    {
        $data = $i_post_card->statusToggle($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }

    public function showReviews(PostCardInterface $i_post_card, PostCardIDRequest $request)
    {
        $data = $i_post_card->showReviews($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }
}
