<?php

namespace App\Models;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Passport\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'email',
        'password',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
        'login_type',
        'login_id',
        'otp',
        'otp_created_at',
        'deleted_at',
        // 'created_at',
        'updated_at',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
        'id' => 'integer',
    ];


    //* Relation function start

    public function favoriteCard()
    {
        return $this->hasMany(FavoriteCard::class, 'user_id')->has('card')->with('card');
    }
    public function addressBooks()
    {
        return $this->hasMany(AddressBook::class, 'user_id');
    }
    public function cardsTop5()
    {
        return $this->hasMany(PostCard::class, 'shop_id')->activeCategory()->inRandomOrder()->limit(6);
    }

    public function getDeviceToken()
    {
        return $this->hasMany(AppNotification::class, 'user_id');
    }

    public function shop()
    {
        return $this->hasOne(Shop::class, 'user_id');
    }

    public function postCards()
    {
        return $this->hasMany(PostCard::class, 'shop_id', 'id');
    }

    public function shopPostCardsWithOrders()
    {
        return $this->hasMany(PostCard::class, 'shop_id', 'id')->withCount('orders');
    }

    public function shopPostCardsWithPendingOrders()
    {
        return $this->hasMany(PostCard::class, 'shop_id', 'id')->withCount('pendingOrders');
    }

    public function shopPostCardsWithSoldOrders()
    {
        return $this->hasMany(PostCard::class, 'shop_id', 'id')->withCount('sold');
    }

    public function customerOrders()
    {
        return $this->hasMany(Order::class, 'user_id');
    }

    public function customerLatestOrders()
    {
        return $this->hasOne(Order::class, 'user_id')->orderBy('id', 'desc')->with('postCard');
    }

    public function userInfo()
    {
        return $this->hasOne(UserInfo::class, 'user_id');
    }

    public function reward_points()
    {
        return $this->hasMany(RewardPoint::class, 'user_id');
    }
    public function pointDetails()
    {
        return $this->hasMany(RewardPoint::class, 'user_id');
    }

    //* Relation function end

    public function scopeCustomer($query)
    {
        $query->where('role', 'customer');
    }

    public function contactUs()
    {
        return $this->hasOne(ContactUs::class, 'user_id');
    }
}
