<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Models\Order;
use App\Models\User;
use App\Models\Voucher;
use App\Services\Interface\VoucherInterface;
use App\Traits\GlobalTrait;
use Exception;
use Illuminate\Support\Facades\Auth;

class VoucherRepository implements VoucherInterface
{
    use GlobalTrait;
    //* All voucher list
    public function voucherList($request)
    {
        try {
            $take = isset($request['take']) ? $request['take'] : 20;
            $all_vouchers = Voucher::paginate($take);
            return Base::pass("All vouchers list", $all_vouchers); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function voucherFilter($request)
    {
        try{
            //Code will be here
            $take = isset($request['take']) ? $request['take'] : 20;
            $search = isset($request['search']) ? $request['search'] : null;
            $all_vouchers = Voucher::when($search != null, function ($q) use ($search){
                $q->where('voucher_code', 'like', '%'.$search.'%')
                ->orWhere('title', 'like', '%' . $search . '%')
                ->orWhere('status',  $search );
            })->paginate($take);

            return Base::pass("Filtered voucher list", $all_vouchers); // DONE
        }
        catch(Exception $e){
            return Base::exception_fail($e);
        }
    }

    public function add($request)
    {
        try {
            // return $request;
            if ($request['type'] == 'global') $user_id = null;
            if ($request['type'] == 'customer') {
                $user_id = $request['user_id'];
                $user = User::find($user_id);
                if (!$user) return Base::fail("User not found"); // DONE
                if ($user->role != 'customer') return Base::fail("User is not customer"); // DONE
            }
            if ($request['type'] == 'shop') {
                $user_id = $request['user_id'];
                $user = User::find($user_id);
                if (!$user) return Base::fail("User not found"); // DONE 
                if ($user->role != 'shop') return Base::fail("User is not shop"); // DONE
            }
            if (isset($request->voucher_code)) $voucher_code = $request['voucher_code'];
            else $voucher_code = GlobalTrait::generateVoucherCode();



            $voucher = new Voucher();

            $voucher->type = $request['type'];
            $voucher->user_id = $user_id;
            $voucher->title = $request['title'];
            $voucher->voucher_code = $voucher_code;
            $voucher->status = 'init';
            $voucher->start_date = Base::carbonParse($request['start_date']);
            $voucher->end_date = Base::carbonParse($request['end_date']);
            $voucher->amount = $request['amount'];
            $voucher->limit = $request['limit'];
            $voucher->no_used = 0;

            $voucher->save();

            // return $voucher;

            return Base::pass("New voucher added"); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function checkVoucher($data)
    {
        try {
            // return $data;
            $voucher_code = $data['voucher_code'];

            $user_id = Auth::guard('api')->user()->id; 

            if(isset($data['user_id'])) $user_id = $data['user_id'];

           $voucher = Voucher::where('voucher_code', $voucher_code)->first();

            $value = true;

            if (!$voucher) {
                $msg = "Voucher is not exist"; // DONE
                $value = false;
                return Base::fail($msg, $value);
            }
            if ($voucher->status == 'expired') {
                $msg = "Voucher is expired"; // DONE
                $value = false;
            }
            if ($voucher->status == 'used') {
                $msg = "Voucher has been used"; // DONE
                $value = false;
            }
            if ($voucher->end_date < Base::now()) {
                $msg = "Voucher is expired"; // DONE
                $value = false;
            }
            if ($voucher->start_date > Base::now()) {
                $msg = "Voucher is not initial yet"; // DONE
                $value = false;
            }
            if ($voucher->no_used == $voucher->limit) {
                $msg = "Voucher limit has been finished"; // DONE
                $value = false;
            }

            if($voucher->is_active == 0){
                $msg = "Voucher does not exist"; // DONE
                $value = false;
                return Base::fail($msg, $value);
            }

            // if($voucher->type == 'global') $value = true;

            if ($voucher->type != 'global') {
                if ($voucher->type == 'shop' && $voucher->user_id != $data['shop_id']) {
                    $msg =  "Invalid voucher"; // DONE
                    $value = false;
                } else if ($voucher->type == 'customer' && $voucher->user_id != $user_id) {
                    $msg =  "Invalid voucher"; // DONE
                    $value = false;
                } else {
                    $msg = "Voucher applied"; // DONE
                    $data = $voucher;
                }
            } else {
                if($value == true){
                    $msg = "Voucher applied"; // DONE
                    $data = $voucher;
                }
            }
            return $value ? Base::pass($msg,$data) : Base::fail($msg, $value);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function voucherApply($data)
    {
        try{
            $voucher_code = $data['voucher_code'];
            $voucher = Voucher::where('voucher_code', $voucher_code)->first();
            if (!$voucher) {
                $msg = "Voucher is not exist"; // DONE
                $value = false;
                return Base::fail($msg, $value);
            }
            $voucher->no_used = $voucher->no_used + 1;
            if(($voucher->no_used + 1) == $voucher->limit){
                $voucher->status = 'used';
            }
            $voucher->save();
        }
        catch(Exception $e){
            return Base::exception_fail($e);
        }
    }

    public function voucherToggle($request)
    {
        try{
            $voucher = Voucher::find($request['id']);

            if(!$voucher) return Base::fail("Voucher not found"); // DONE

            $voucher->is_active = $request['is_active'] == 1 ? 1 : 0;
            $voucher->save();

            return  Base::pass("Voucher status toggled successfully"); // DONE
        }
        catch(Exception $e){
            return Base::exception_fail($e);
        }
    }

    public function voucherDelete($data)
    {
        try{
            $voucher = Voucher::find($data['id']);
            if (!$voucher) return Base::fail("Voucher not found");
            if($voucher->no_used > 0) return Base::fail("This voucher has been used more then once"); // DONE

            $voucher->delete();

            $voucher_list = $this->voucherList($data);
            $data = $voucher_list->data;

            return Base::pass("Voucher has been deleted successfully", $data); // DONE
        }
        catch(Exception $e){
            return Base::exception_fail($e);
        }
    }

    public function voucherDetails($request)
    {
        try{
            $voucher = Voucher::find($request['id']);
            if (!$voucher) return Base::fail("Voucher not found"); // DONE

            $voucher_orders = Order::where(['discount_code' => $voucher->voucher_code])->get();

            $orders = [];

            foreach($voucher_orders as $item){
                $order = [
                    "user_id" => $item->customer->id,
                    "user_name" => $item->customer->name,
                    "order_id" => $item->id,
                    "card_title" => $item->card_name,
                    "ordered_at" => $item->created_at,
                ];
                array_push($orders,$order);
            }
            
            $data = [
                "voucher" => $voucher,
                "voucher_orders" => $orders,
            ];

            return Base::pass("Voucher details", $data); // DONE
        }
        catch(Exception $e){
            return Base::exception_fail($e);
        }
    }
}
